from typing import List, Tuple

from .parser import Parser


class Vectorizer:
    def __init__(self):
        self.parser = Parser()
        self.BOS = '<s>'
        self.EOS = '</s>'

    def make_word_ngram(self, text: str, n: int = 2, bos: str = None, eos: str = None) -> List[List[Tuple[str]]]:
        """
        Make word ngrams.

        :param text: input text
        :param n: an integer
        :param bos: a string for the beginning of sentence
        :param eos: a string for the end of sentence
        :return: List[List[Tuple]]

        example
        -------

        input: '今日はいい天気ですね。どちらへ行くのです？'
        output (2gram):
        [
            [('今日', 'は'), ('は', 'いい'), ('いい', '天気'), ('天気', 'です'), ('です', 'ね'), ('ね', '。')],
            [('どちら', 'へ'), ('へ', '行く'), ('行く', 'の'), ('の', 'です'), ('です', '？')]
        ]
        """

        assert n > 0, f'n={n} must larger than 0.'
        assert type(n) == int, f'n={n} must be an integer value.'

        sents_surfaces = [ self.parser.get_surfaces(sent) for sent in self.parser.split_to_lines(text) ]
        # sents_surfaces = [ ['w1', 'w2'], ['w3', 'w4'] ]
        if bos:
            # sents_surfaces = [ ['<bos>', 'w1', 'w2'], ['<bos>', 'w3', 'w4'] ]
            sents_surfaces = [ [bos] + surfs for surfs in sents_surfaces ]
        if eos:
            # sents_surfaces = [ ['w1', 'w2', '<eos>'], ['w3', 'w4', '<eos>'] ]
            sents_surfaces = [ surfs + [eos] for surfs in sents_surfaces ]

        if n == 1:
            sents_ngrams = [ [ tuple(surf) for surf in surfs ] for surfs in sents_surfaces ]
            # sents_ngrams = [ tuple(surfs) for surfs in sents_surfaces ]
        else:
            sents_ngrams = []
            for surfs in sents_surfaces:
                sent_length = len(surfs)

                assert n <= sent_length, f'n={n} must smaller than sentence length.'

                sents_ngrams.append([ tuple([ surfs[i+j] for j in range(n) ])
                                      for i in range(sent_length - n + 1) ])

        return sents_ngrams

    def make_char_ngram(self, text: str, n: int = 2) -> List[List[Tuple[str]]]:
        """
        Make char ngrams.

        :param text: input text
        :param n: an integer
        :return: List[List[Tuple]]

        example
        -------

        input: '今日はいい天気ですね。どちらへ行くのです？'
        output (2gram):
        [
            [('今', '日'), ('日', 'は'), ('は', 'い'), ('い', 'い'), ('い', '天'),
             ('天', '気'), ('気', 'で'), ('で', 'す'), ('す', 'ね'), ('ね', '。')],
            [('ど', 'ち'), ('ち', 'ら'), ('ら', 'へ'), ('へ', '行'), ('行', 'く'),
             ('く', 'の'), ('の', 'で'), ('で', 'す'), ('す', '？')]
        ]
        """

        assert n > 0, f'n={n} must larger than 0.'
        assert type(n) == int, f'n={n} must be an integer value.'

        sentences = self.parser.split_to_lines(text)

        if n == 1:
            sents_ngrams = [ [ char for char in sent ] for sent in sentences ]
        else:
            sents_ngrams = []
            for sent in sentences:
                sent_length = len(sent)

                assert n <= sent_length, f'n={n} must smaller than sentence length.'

                sents_ngrams.append([ tuple([ sent[i+j] for j in range(n) ])
                                      for i in range(sent_length - n + 1) ])

        return sents_ngrams



from pathlib import Path

import matplotlib
from matplotlib import font_manager
from distutils.version import LooseVersion


def init_matplotlib(weight='medium'):
    """
    Initialize matplotlib settings.
    :param weight: you can choose one weight from 'thin', 'light', 'regular', 'medium', 'bold', 'black'.
                   default is 'medium'
    :return:
    """
    FONTS_DIR = 'fonts'
    FONT_NAME = 'Noto Sans CJK JP'
    FONT_WEIGHT = weight

    font_settings = {
        'family': FONT_NAME,
        'weight': FONT_WEIGHT
    }

    font_dir_path = Path(__file__).parent.resolve() / Path(FONTS_DIR)
    font_dirs = [font_dir_path, ]
    font_files = font_manager.findSystemFonts(fontpaths=font_dirs)

    is_support_createFontList = LooseVersion(matplotlib.__version__) < '3.2'
    if is_support_createFontList:
        font_list = font_manager.createFontList(font_files)
        font_manager.fontManager.ttflist.extend(font_list)
    else:
        for fpath in font_files:
            font_manager.fontManager.addfont(fpath)
    matplotlib.rc('font', **font_settings)



import re
from typing import List

import MeCab
import ipadic


class Parser:
    def __init__(self) -> None:
        self.feature_columns = [
            '品詞', '品詞細分類1', '品詞細分類2', '品詞細分類3',
            '活用型', '活用形', '基本形', '読み', '発音',
        ]
        self.stopwords = []
        self.content_pos = ['名詞', '動詞', '形容詞', '副詞', '連体詞']
        self.eos = ['。', '？', '！', '．', '\?', '\!']
        self.ptn_eos = re.compile(f'({"|".join(self.eos)})')

        self.mecab_args = ipadic.MECAB_ARGS
        self.tagger = MeCab.Tagger(self.mecab_args)

    def set_args(self, mecab_args: str, is_dict_path: bool = False) -> None:
        """
        Specify the dictionary path

        :param mecab_args:
            Ubuntu Env Example
                IPADic: See https://github.com/polm/ipadic-py#usage
                Neologd: -r/dev/null -d /usr/lib/mecab/dic/mecab-ipadic-neologd
        :param is_dict_path:
            activate if the mecab_args is dict path
        :return:
        """

        if is_dict_path:
            self.mecab_args = '-r/dev/null -d ' + mecab_args

        self.tagger = MeCab.Tagger(self.mecab_args)

    def set_stopwords(self, stopwords: list) -> None:
        self.stopwords = stopwords

    def set_content_pos(self, content_pos: list) -> None:
        self.content_pos = content_pos

    def set_eos(self, eos: list) -> None:
        self.eos = eos

    def parse(self, text: str, only_content_words: bool = False, fit_stopwords: bool = False) -> List[dict]:
        results = []
        for node in self.tagger.parse(text).rstrip('\n').splitlines():
            if node == 'EOS' or node == '':
                continue

            surface, feature = node.split('\t')
            features = feature.split(',')

            if only_content_words:
                if features[0] in self.content_pos:
                    results.append({'surface': surface, 'features': features})
            elif fit_stopwords:
                if surface not in self.stopwords:
                    results.append({'surface': surface, 'features': features})
            else:
                results.append({'surface': surface, 'features': features})

        return results

    def get_surfaces(self, text: str, only_content_words: bool = False, fit_stopwords: bool = False) -> List[str]:
        return [ res['surface'] for res in self.parse(text,
                                                      only_content_words=only_content_words,
                                                      fit_stopwords=fit_stopwords) ]

    def split_to_lines(self, text: str) -> List[str]:
        return re.sub(self.ptn_eos, '\g<1>\n', text.replace('\n', '').rstrip()).splitlines()


if __name__ == '__main__':
    from pathlib import Path
    from icecream import ic
    sample_lines = (Path(__file__).parent / '../sample.txt').open('r').readlines()
    sample_text = 'こんにちは！今日はいい天気ですね。これからどちらへ？すもももももももものうち'

    p = Parser()
    ic(sample_text)
    ic(p.parse(sample_text))
    ic(p.get_surfaces(sample_text))
    ic(p.get_surfaces(sample_text, only_content_words=True))
    ic(p.split_to_lines(sample_text))

    p.set_args('/usr/local/lib/mecab/dic/mecab-ipadic-neologd', is_dict_path=True)
    ic(p.get_surfaces(sample_text))



from .parser import Parser

from typing import Union
from pathlib import Path

from itertools import chain
from collections import Counter


class Statist:
    def __init__(self, inputs: Union[str, list]):
        self.corpus: list = self.make_corpus(inputs)
        self.corpus_str: str = ''.join(self.corpus)
        self.parser = Parser()

    def make_corpus(self, inputs: Union[str, list]) -> list:
        if type(inputs) == list:
            corpus = inputs
        elif type(inputs) == str:
            if Path(inputs).exists():
                corpus = Path(inputs).open('r').readlines()
            else:
                corpus = inputs.splitlines()
        else:
            raise TypeError(f'only `str` or `list` type is valid, but {type(inputs)} was given.')

        return corpus

    def all_stats(self, verbose=False) -> dict:
        word_stats = self.calc_word_stats(verbose)
        char_stats = self.calc_char_stats(verbose)

        return dict(
            word_stats=word_stats,
            char_stats=char_stats
        )

    def calc_word_stats(self, verbose=False) -> dict:
        c = Counter(self.parser.get_surfaces(self.corpus_str))

        if verbose:
            print(c)

        return dict(
            num_token=sum(c.values()),
            num_vocab=len(c.keys())
        )

    def calc_char_stats(self, verbose=False) -> dict:
        c = Counter(self.corpus_str)

        if verbose:
            print(c)

        return dict(
            num_token=sum(c.values()),
            num_vocab=len(c.keys())
        )



import sys
from loguru import logger


def get_logger(sink=sys.stderr, level='DEBUG') -> logger:
    logger.level('TRACE',    icon='TRA')
    logger.level('DEBUG',    icon='DBG')
    logger.level('INFO',     icon='INF')
    logger.level('SUCCESS',  icon='SCS')
    logger.level('WARNING',  icon='WRN')
    logger.level('ERROR',    icon='ERR')
    logger.level('CRITICAL', icon='CRT')

    log_format = (
        '<level>{level.icon}</level> <b>{time:YYMMDDtHHmmss}</b>'
        ' | <cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan>'
        ' | <level>{message}</level>'
    )

    logger.remove()
    logger.add(
        sink, format=log_format,
        filter=None, colorize=None, serialize=False, backtrace=True,
        diagnose=True, enqueue=False, catch=True, level=level
    )

    return logger


if __name__ == '__main__':
    l = get_logger()
    levels = ['TRACE', 'DEBUG', 'INFO', 'SUCCESS', 'WARNING', 'ERROR', 'CRITICAL']
    for level in levels:
        l.log(level, f'This is {level} level message.')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['peinan_utils']

package_data = \
{'': ['*'], 'peinan_utils': ['fonts/*']}

install_requires = \
['ipadic>=1.0.0,<2.0.0',
 'loguru>=0.5.3,<0.6.0',
 'matplotlib>=3.4.2,<4.0.0',
 'mecab-python3>=1.0.4,<2.0.0']

setup_kwargs = {
    'name': 'peinan-utils',
    'version': '0.1.0',
    'description': 'An util package for myself. (Mostly about NLP)',
    'long_description': "# An util package for myself\n\n![](https://img.shields.io/badge/version-0.0.3-green.svg?style=flat-square) ![](https://img.shields.io/badge/python-3.6+-green.svg?style=flat-square) ![](https://img.shields.io/badge/license-MIT-blue.svg?style=flat-square)\n\nMostly about NLP.\n\n## Install\n\n```bash\n$ pip install git+https://github.com/peinan/peinan-utils-py\n```\n\n## Usages & Features\n\n### Parser\n\n```python\n# sample text\n>>> text = 'こんにちは！今日はいい天気ですね。これからどちらへ？'\n\n>>> from peinan_utils import Parser\n>>> p = Parser()\n\n# just parse input texts and get information about surfaces and features\n>>> p.parse(text)\n[{'surface': 'こんにちは',\n  'features': ['感動詞', '*', '*', '*', '*', '*', 'こんにちは', 'コンニチハ', 'コンニチワ']},\n {'surface': '！', 'features': ['記号', '一般', '*', '*', '*', '*', '！', '！', '！']},\n {'surface': '今日',\n  'features': ['名詞', '副詞可能', '*', '*', '*', '*', '今日', 'キョウ', 'キョー']},\n {'surface': 'は',\n  'features': ['助詞', '係助詞', '*', '*', '*', '*', 'は', 'ハ', 'ワ']},\n {'surface': 'いい',\n  'features': ['形容詞', '自立', '*', '*', '形容詞・イイ', '基本形', 'いい', 'イイ', 'イイ']},\n {'surface': '天気',\n  'features': ['名詞', '一般', '*', '*', '*', '*', '天気', 'テンキ', 'テンキ']},\n {'surface': 'です',\n  'features': ['助動詞', '*', '*', '*', '特殊・デス', '基本形', 'です', 'デス', 'デス']},\n {'surface': 'ね',\n  'features': ['助詞', '終助詞', '*', '*', '*', '*', 'ね', 'ネ', 'ネ']},\n {'surface': '。', 'features': ['記号', '句点', '*', '*', '*', '*', '。', '。', '。']},\n {'surface': 'これから',\n  'features': ['副詞', '助詞類接続', '*', '*', '*', '*', 'これから', 'コレカラ', 'コレカラ']},\n {'surface': 'どちら',\n  'features': ['名詞', '代名詞', '一般', '*', '*', '*', 'どちら', 'ドチラ', 'ドチラ']},\n {'surface': 'へ',\n  'features': ['助詞', '格助詞', '一般', '*', '*', '*', 'へ', 'ヘ', 'エ']},\n {'surface': '？', 'features': ['記号', '一般', '*', '*', '*', '*', '？', '？', '？']}]\n\n# get surfaces of input text\n>>> p.get_surfaces(text)\n['こんにちは', '！', '今日', 'は', 'いい', '天気', 'です', 'ね', '。', 'これから', 'どちら', 'へ', '？']\n\n# get only content words of input text\n>>> p.get_surfaces(text, only_content_words=True)\n['今日', 'いい', '天気', 'これから', 'どちら']\n\n# split input text into lines\n>>> p.split_to_lines(text)\n['こんにちは！', '今日はいい天気ですね。', 'これからどちらへ？']\n\n# change dictonary path\n>>> sumomo = 'すもももももももものうち'\n>>> p.get_surfaces(sumomo)\n['すもも', 'も', 'もも', 'も', 'もも', 'の', 'うち']\n\n>>> p.set_dict('/usr/local/lib/mecab/dic/mecab-ipadic-neologd', is_dict_path=True)\n>>> p.get_surfaces(sumomo)\n['すもももももももものうち']\n```\n\n### Vectorizer\n\n```python\n# sample text\n>>> text = '今日はいい天気ですね。これからどちらへ？'\n\n>>> from peinan_utils import Vectorizer\n>>> v = Vectorizer()\n\n# make word ngram (the default n is 2)\n>>> v.make_word_ngram(text)\n[[('今日', 'は'), ('は', 'いい'), ('いい', '天気'), ('天気', 'です'), ('です', 'ね'), ('ね', '。')], [('これから', 'どちら'), ('どちら', 'へ'), ('へ', '？')]]\n\n# specify n\n>>> v.make_word_ngram(text, n=3)\n[[('今日', 'は', 'いい'), ('は', 'いい', '天気'), ('いい', '天気', 'です'), ('天気', 'です', 'ね'), ('です', 'ね', '。')], [('これから', 'どちら', 'へ'), ('どちら', 'へ', '？')]]\n\n# use BOS and EOS (the default BOS and EOS are '<s>' and </s>, respectively)\n>>> v.make_word_ngram(text, n=3, bos=v.BOS, eos=v.EOS)\n[[('<s>', '今日', 'は'), ('今日', 'は', 'いい'), ('は', 'いい', '天気'), ('いい', '天気', 'です'), ('天気', 'です', 'ね'), ('です', 'ね', '。'), ('ね', '。', '</s>')], [('<s>', 'これから', 'どちら'), ('これから', 'どちら', 'へ'), ('どちら', 'へ', '？'), ('へ', '？', '</s>')]]\n\n# make character ngram (the default n is 2)\n>>> v.make_char_ngram(text, n=2)\n[[('今', '日'), ('日', 'は'), ('は', 'い'), ('い', 'い'), ('い', '天'), ('天', '気'), ('気', 'で'), ('で', 'す'), ('す', 'ね'), ('ね', '。')], [('こ', 'れ'), ('れ', 'か'), ('か', 'ら'), ('ら', 'ど'), ('ど', 'ち'), ('ち', 'ら'), ('ら', 'へ'), ('へ', '？')]]\n```\n\n### Statist\n\nsample.txt\n\n```\nあのイーハトーヴォのすきとおった風、夏でも底に冷たさをもつ青いそら、うつくしい森で飾られたモリーオ市、 郊外のぎらぎらひかる草の波。\nまたそのなかでいっしょになったたくさんのひとたち、ファゼーロとロザーロ、羊飼のミーロや、顔の赤いこども たち、地主のテーモ、山猫博士のボーガント・デストゥパーゴなど、いまこの暗い巨きな石の建物のなかで考えて いると、みんなむかし風のなつかしい青い幻燈のように思われます。\nでは、わたくしはいつかの小さなみだしをつけながら、しずかにあの年のイーハトーヴォの五月から十月までを書 きつけましょう。\n```\n\n```python\nIn : from peinan_utils import Statist\nIn : s = Statist('./sample.txt')\n# you can put data directly such as below\n# In : s = Statist('あのイーハトーヴォのすきとおった風、夏でも底に冷たさをもつ青いそら、うつくしい森で飾られたモリーオ市、 郊外のぎらぎらひかる草の波。')\n# In : s = Statist([\n#          'あのイーハトーヴォのすきとおった風、夏でも底に冷たさをもつ青いそら、うつくしい森で飾られたモリーオ市、 郊外のぎらぎらひかる草の波。',\n#          'またそのなかでいっしょになったたくさんのひとたち、ファゼーロとロザーロ、羊飼のミーロや、顔の赤いこども たち、地主のテーモ、山猫博士のボーガント・デストゥパーゴなど、いまこの暗い巨きな石の建物のなかで考えて いると、みんなむかし風のなつかしい青い幻燈のように思われます。',\n#          'では、わたくしはいつかの小さなみだしをつけながら、しずかにあの年のイーハトーヴォの五月から十月までを書 きつけましょう。'\n#      ])\n\n# show all stats\nIn : s.all_stats()\nOut: {'word_stats': {'num_token': 137, 'num_vocab': 90},  # num_token == 延べ語数, num_vocab == 異なり語数\n      'char_stats': {'num_token': 260, 'num_vocab': 109}}\n \n# show word stats with verbose mode\nIn : s.calc_word_stats(verbose=True)\nCounter({'の': 15, '、': 12, 'で': 5, 'に': 4, 'た': 3, 'を': 3, '。': 3, 'あの': 2, 'イーハトーヴォ': 2, '風': 2, '青い': 2, 'れ': 2, 'なか': 2, 'たち': 2, 'と': 2, 'は': 2, 'すきとおっ': 1, '夏': 1, 'も': 1, '底': 1, '冷た': 1, 'さ': 1, 'もつ': 1, 'そら': 1, 'うつくしい': 1, '森': 1, '飾ら': 1, 'モリーオ': 1, '市': 1, '郊外': 1, 'ぎらぎら': 1, 'ひかる': 1, '草': 1, '波': 1, 'また': 1, 'その': 1, 'いっしょ': 1, 'なっ': 1, 'たくさん': 1, 'ひと': 1, 'ファゼーロ': 1, 'ロザーロ': 1, '羊': 1, '飼': 1, 'ミーロ': 1, 'や': 1, '顔': 1, '赤い': 1, 'こども': 1, '地主': 1, 'テーモ': 1, '山猫': 1, '博士': 1, 'ボーガント・デスト ゥパーゴ': 1, 'など': 1, 'いま': 1, 'この': 1, '暗い': 1, '巨': 1, 'き': 1, 'な': 1, '石': 1, '建物': 1, '考え': 1, 'て': 1, 'いる': 1, 'みんな': 1, 'むかし': 1, 'なつかしい': 1, '幻': 1, '燈': 1, 'よう': 1, '思わ': 1, 'ます': 1, 'わたくし': 1, 'いつか': 1, '小さな': 1, 'み': 1, 'だし': 1, 'つけ': 1, 'ながら': 1, 'しずか': 1, '年': 1, '五月': 1, 'から': 1, '十月': 1, 'まで': 1, '書きつけ': 1, 'ましょ': 1, 'う': 1})\nOut: {'num_token': 137, 'num_vocab': 90}\n```\n\n### Matplotlib Utils\n\n```python\n# just import this package and then you can plot with Japanese font\n>>> import matplotlib.pyplot as plt\n>>> import peinan_utils\n\n# set background face color\n>>> fig = plt.figure()\n>>> fig.patch.set_facecolor('white')\n\n# plot with Japanese labels\n>>> plt.plot([1,2,3], [1,2,3])\n>>> plt.xlabel('x軸')\n>>> plt.ylabel('y軸')\n>>> plt.show()\n```\n\n![](https://raw.githubusercontent.com/peinan/peinan-utils-py/master/images/plot_result.png)\n",
    'author': 'Peinan ZHANG',
    'author_email': 'peinan7@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/peinan/peinan-utils-py',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



__version__ = '0.2.0'

from .parser import Parser
from .vectorizer import Vectorizer
from .stat_utils import Statist
from .matplotlib_utils import init_matplotlib
from .logger import get_logger
init_matplotlib()
